<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
</div>
<script type="text/menu">
[{
  "title": "sendTestMail",
  "action": "sendTestMail",
  "icon": "envelope"
}]
</script>
