<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <div class="panel-body">
    <div class="row">
      <div class="col-xs-12 col-sm-12 col-md-6 col-lg-6">
        <div class="form-group">
          <label for="keyid" id="lki"><span>{{translateTitle(currentNode.data[2])}}</span></label>
          <input id="keyid" aria-describedby="lki" class="form-control" ng-model="currentNode.data[2].data"/>
        </div>
        <div class="form-group">
          <label for="privateKey" id="lpv"><span>{{translateTitle(currentNode.data[0])}}</span></label>
          <textarea id="privateKey" aria-describedby="lpv" class="form-control" rows="8" ng-model="currentNode.data[0].data"></textarea>
        </div>
        <div class="form-group">
          <label for="privkfile" id="lprivkfile"><span trspan="replaceByFile"></span></label>
          <input id="privkfile" aria-describedby="lprivkfile" type="file" class="form-control" on-read-file="replaceContent(currentNode.data[0],$fileContent)"/>
        </div>
      </div>
      <div class="col-xs-12 col-sm-12 col-md-6 col-lg-6">
	<div class="form-group">
	  <label for="selform" id="selLabel"><span>{{translateTitle(currentNode.data[3])}}</span></label>
	  <select id="selform" class="form-control" ng-model="currentNode.data[3].data" ng-change="currentNode.onChange()">
	    <option ng-selected="'RSA'==currentNode.data[3].data" value="RSA">RSA</option>
	    <option ng-selected="'EC'==currentNode.data[3].data" value="EC">EC</option>
	  </select>
	</div>
        <div class="form-group">
          <label for="publicKey" id="lpub"><span>{{translateTitle(currentNode.data[1])}}</span></label>
          <textarea id="publicKey" aria-describedby="lpub" class="form-control" rows="8" ng-model="currentNode.data[1].data"></textarea>
        </div>
        <div class="form-group">
          <label for="privkfile" id="lprivkfile"><span trspan="replaceByFile"></span></label>
          <input id="privkfile" aria-describedby="lprivkfile" type="file" class="form-control" on-read-file="replaceContent(currentNode.data[1],$fileContent)"/>
        </div>
      </div>
    </div>
    <hr/>
    <div class="row" ng-if="currentNode.data.length>3">
      <div class="col-xs-12 col-sm-12 col-md-6 col-lg-6">
        <div class="form-group">
          <label for="privateKey2" id="lpv2"><span>{{translateTitle(currentNode.data[3])}}</span></label>
          <textarea id="privateKey2" aria-describedby="lpv2" class="form-control" rows="8" ng-model="currentNode.data[3].data" disabled></textarea>
        </div>
        <div class="form-group">
          <label for="keyid2" id="lki2"><span>{{translateTitle(currentNode.data[5])}}</span></label>
          <input id="keyid2" aria-describedby="lki2" class="form-control" ng-model="currentNode.data[5].data" disabled/>
        </div>
      </div>
      <div class="col-xs-12 col-sm-12 col-md-6 col-lg-6">
        <div class="form-group">
          <label for="publicKey2" id="lpub2"><span>{{translateTitle(currentNode.data[4])}}</span></label>
          <textarea id="publicKey2" aria-describedby="lpub2" class="form-control" rows="8" ng-model="currentNode.data[4].data" disabled></textarea>
        </div>
      </div>
    </div>
    <hr/>
    <div class="row" ng-if="currentNode.data.length>6">
      <div class="col-xs-12 col-sm-12 col-md-6 col-lg-6">
        <div class="form-group">
          <label for="privateKey3" id="lpv3"><span>{{translateTitle(currentNode.data[6])}}</span></label>
          <textarea id="privateKey3" aria-describedby="lpv3" class="form-control" rows="8" ng-model="currentNode.data[6].data"></textarea>
        </div>
        <div class="form-group">
          <label for="keyid3" id="lki3"><span>{{translateTitle(currentNode.data[8])}}</span></label>
          <input id="keyid3" aria-describedby="lki3" class="form-control" ng-model="currentNode.data[8].data"/>
        </div>
      </div>
      <div class="col-xs-12 col-sm-12 col-md-6 col-lg-6">
        <div class="form-group">
          <label for="publicKey3" id="lpub3"><span>{{translateTitle(currentNode.data[7])}}</span></label>
          <textarea id="publicKey3" aria-describedby="lpub3" class="form-control" rows="8" ng-model="currentNode.data[7].data"></textarea>
        </div>
      </div>
    </div>
  </div>
</div>
<script type="text/menu">
[{
 "title": "newCertificate",
 "action": "newCertificateNoPassword",
 "icon": "plus-sign"
},{
 "title": "newEcKeys",
 "action": "newEcKeys",
 "icon": "plus-sign"
},{
  "title": "download",
  "icon": "save-file",
  "action": "saveAsPem"
}]
</script>
