<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title" trspan="authChain"></h3>
  </div>
  <table class="table table-striped">
    <tbody>
      <tr>
        <td id="namelabel" for="namei" trspan="name"></td>
        <td>
          <input class="form-control" ng-model="currentNode.title" id="namei" aria-describedby="namelabel">
        </td>
      </tr>
      <tr ng-repeat="i in [[0,'authentication'],[1,'userDB'],[2,'passwordDB']]">
        <td trspan="{{i[1]}}" id="{{i[1]+'label'}}" for="{{i[1]+'selectform'}}"></td>
        <td>
          <select id="{{i[1]+'selectform'}}" class="form-control" ng-model="currentNode.data[i[0]]" ng-change="execFilters(_findScopeByKey('authParams'))" aria-describedby="{{i[1]+'label'}}">
            <option ng-repeat="item in _findContainer().select[i[0]]" ng-selected="item.k==currentNode.data[i[0]]" value="{{item.k}}" trspan="{{item.v}}">{{item.v}}</option>
          </select>
        </td>
      </tr>
      <tr>
        <td id="urllabel" for="urli" trspan="url"></td>
        <td>
          <input class="form-control" ng-model="currentNode.data[3]" id="urli" aria-describedby="urllabel">
        </td>
      </tr>
      <tr>
        <td id="condlabel" for="condi" trspan="condition"></td>
        <td>
          <input class="form-control" ng-model="currentNode.data[4]" id="condi" aria-describedby="condlabel">
        </td>
      </tr>
    </tbody>
  </table>
  <h4 trspan="overPrm"></h4>
  <table class="table">
    <tr ng-repeat="t in currentNode.data[5]">
      <td>
        <input class="form-control" ng-model="t[0]" />
      </td>
      <td>
        <input class="form-control" ng-model="t[1]" />
      </td>
      <td>
        <span class="link text-danger glyphicon glyphicon-minus-sign" ng-click="del(currentNode.data[5],$index)"></span>
        <span ng-if="$last" class="link text-success glyphicon glyphicon-plus-sign"
              ng-click="menuClick({title:'newCmbOver', action:'newChoiceOver'})"></span>
      </td>
    </tr>
  </table>
</div>
<script type="text/menu">
[{
  "action": "newChoiceOver",
  "title": "newCmbOver",
  "icon": "plus-sign"
},{
  "title": "deleteEntry",
  "icon": "minus-sign"
}]
</script>
