<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <table class="table table-striped">
    <thead>
      <tr>
        <th width="20%" trspan="comments"></th>
        <th width="30%" trspan="rules"></th>
        <th width="50%" trspan="messages"></th>
        <th></th>
      </tr>
    </thead>
    <tbody>
      <tr ng-repeat="s in currentNode.nodes">
        <td>
          <input class="form-control" ng-model="s.comment" ng-change="changeRuleTitle(s)"/>
        </td>
        <td>
          <input class="form-control" ng-model="s.data"/>
        </td>
        <td>
          <input class="form-control" ng-model="s.re" ng-change="changeRuleTitle(s)"/>
        </td>
        <td>
          <span ng-model="s.title" class="link text-danger glyphicon glyphicon-minus-sign" ng-click="del(currentNode.nodes,$index)"></span>
          <span ng-if="$last" class="link text-success glyphicon glyphicon-plus-sign" ng-click="menuClick({title:'newGrantRule',action:'newGrantRule'})"></span>
        </td>
      </tr>
    </tbody>
  </table>
</div>
<script type="text/menu">
[{
  "title": "newGrantRule",
  "action": "newGrantRule",
  "icon": "plus-sign"
}]
</script>
