#!/usr/bin/perl -i -p

s/â/&acirc;/g;
s/à/&agrave;/g;
s/é/&eacute;/g;
s/ê/&ecirc;/g;
s/è/&egrave;/g;
s/ë/&euml;/g;
s/î/&icirc;/g;
s/ï/&iuml;/g;
s/ô/&ocirc;/g;
s/œ/&oelig;/g;
s/û/&ucirc;/g;
s/ù/&ugrave;/g;
s/ü/&uuml;/g;
s/ç/&ccedil;/g;
