    <!-- Custom HTML header -->
<TMPL_IF NAME="MAIN_LOGO">
    <img src="<TMPL_VAR NAME="STATIC_PREFIX"><TMPL_VAR NAME="MAIN_LOGO">" alt="" class="img-fluid rounded mx-auto d-block" />
</TMPL_IF>
<TMPL_IF NAME="iniParam"><p class="iniParam"><TMPL_VAR NAME="iniParam"></p></TMPL_IF>
<TMPL_IF NAME="confParam"><p class="confParam"><TMPL_VAR NAME="confParam"></p></TMPL_IF>

