/* password entropy customization*/
#entropybar>div {
  width: 0%;
  /* Adjust with JavaScript */
}

#entropybar>div.levelErr {
  width: 0%;
}

#entropybar>div.level0 {
  width: 20%;
}

#entropybar>div.level1 {
  width: 40%;
}

#entropybar>div.level2 {
  width: 60%;
}

#entropybar>div.level3 {
  width: 80%;
}

#entropybar>div.level4 {
  width: 100%;
}

.entropyHidden {
  display: none;
}
