<script type="text/javascript" src="<TMPL_VAR NAME="js">""></script>
<script type="application/init">
{"sessions":<TMPL_VAR NAME="sessions">}
</script>
<div class="card border-secondary">
  <div class="card-header text-white bg-secondary">
    <h4 class="card-title" trspan="oidcOfflineTokens">Oidc Offline tokens</h4>
  </div>
  <div class="card-body buttons">
    <table class="info">
      <thead>
        <tr>
        <th trspan="service">Client ID</th>
        <th trspan="updateTime">Update Time</th>
        <th></th>
        </tr>
      </thead>
      <tbody class="oidcOfflineTokens">
      </tbody>
      </table>
    </div>
  </div>
</div>