  <!-- Custom HTML footer -->
    <p>
      <span trspan="serviceProvidedBy">Service provided by</span>
      <a href="https://lemonldap-ng.org" target="_blank" class="btn btn-secondary btn-sm">
        <span class="fa fa-globe"></span>
        LemonLDAP::NG
      </a>
      <span trspan="gplSoft">free software covered by the GPL license.</span>
    </p>
